# -*- coding: utf-8 -*-
# N guage control panel by ik-gadgets
# Network interface = Bluetooth Low Energy

import argparse
import asyncio
import re
from bleak import BleakScanner
from bleak import BleakClient
import sys
import os
import time
from multiprocessing import Process,Queue

from ikcp_gui2 import ikcp_gui

txrx_interval = 0.1 # minimum period of Tx/Rx loop (sec)

##### BLE #####
address = ""
#### Circuit Board: ESP32C3toSNTC ####
ServiceUUID = "6eea8079-7429-405d-a9cf-87bb56f1e003"
TxUUID = "52506b1f-01e3-4e83-991f-cacf437af658"
RxUUID = "0a7151b3-d8e6-47bb-b615-e21d0ff1e681"

async def get_BLE_address(args: argparse.Namespace):
    global address
    print("scanning for 5 seconds, please wait...\n")

    devices = await BleakScanner.discover(return_adv=True)
#    devices = await BleakScanner.discover(
#        return_adv=True, cb=dict(use_bdaddr=args.macos_use_bdaddr)
#    )

    for d, a in devices.values():
        print(a)
        L = str(d).split()
        if L[1] == "SNTC":
            print("Found SNTC! , address= ",L[0])
            print()
            address = L[0]
            address = re.sub(r':$','',address)
            return
    print("### ERROR #### Bluetooth SNTC not found")
    sys.exit()

async def BLE_TxRx(address, loop, qtx, qrx):
    #### Connect ####
    async with BleakClient(address, loop=loop) as client:
        x = client.is_connected
        print("Connected: {0}\n".format(x))
        print("")

        #### dummy BLE TxRx on initial to avoid error ####
        tx_data = []
        for i in range(200):
            tx_data.append(254)
        tx_data.append(255)
        tx_data = bytearray(tx_data)
        time.sleep(0.5)
        await client.write_gatt_char(TxUUID, tx_data)
        rx_data = await client.read_gatt_char(RxUUID)
        #### dummy BLE TxRx end ####
            
        while(True):
            if qtx.qsize() != 0:
                tx_data = qtx.get()
                if tx_data == [255,255]: # disconnect flag from GUI
                    await client.unpair()
                    result = await client.disconnect()
                    print("disconnect end",result)
                    break
                else:
#                    print("DEB:tx_data=",tx_data)
                    tx_data = bytearray(tx_data)
                    await client.write_gatt_char(TxUUID, tx_data)

                rx_data = await client.read_gatt_char(RxUUID)
                qrx.put(rx_data)

def main():
#    start = time.time()

    args = sys.argv
    panel_file_list = []
    auto_prg_file_list = []
    com_print_flag = 0
    beep_flag = 0

    if len(sys.argv) >= 2:
        for file_name in sys.argv[1:]:
            if re.search(r'\.txt$',file_name):
                if os.path.exists(file_name):
                    panel_file_list.append(file_name)
                else:
                    print("### ERROR ### file",file_name,"not found")
                    print()
                    sys.exit()
            elif re.search(r'\.py$',file_name):
                if os.path.exists(file_name):
                    auto_prg_file_list.append(file_name)
                else:
                    print("### ERROR ### file",file_name,"not found")
                    print()
                    sys.exit()
            elif re.search(r'^\-print$',file_name): # print auto_read/write to com window
                com_print_flag = 3
            elif re.search(r'^\-print_r',file_name): # print auto_read/write to com window
                com_print_flag = 1
            elif re.search(r'^\-print_w',file_name): # print auto_read/write to com window
                com_print_flag = 2
            elif re.search(r'^\-beep$',file_name): # beep on auto_read/write
                beep_flag = 3
            elif re.search(r'^\-beep_r',file_name): # beep on auto_read/write
                beep_flag = 1
            elif re.search(r'^\-beep_w',file_name): # beep on auto_read/write
                beep_flag = 2
            else:
                print("### ERROR ###",file_name,"? argument must be *.txt or *.py")
                print()
                sys.exit()
    
    qtx = Queue()
    qrx = Queue()

#    parser = argparse.ArgumentParser()
#    parser.add_argument(
#        "--macos-use-bdaddr",
#        action="store_true",
#        help="when true use Bluetooth address instead of UUID on macOS",
#    )
#    args = parser.parse_args()
#    print("DEB args",args)
#    sys.exit()

    asyncio.run(get_BLE_address(args))

    process = Process(target=ikcp_gui,args=(qtx,qrx,panel_file_list,auto_prg_file_list,com_print_flag,beep_flag))
    process.start()

    loop = asyncio.new_event_loop()
    loop.run_until_complete(BLE_TxRx(address, loop, qtx, qrx))


    process.join()

if __name__ == '__main__':
    main()
