# -*- coding: utf-8 -*-
# N guage control panel by ik-gadgets
# Network interface = Serial com port

import sys
import re
import os
import serial
import serial.tools.list_ports
import time
from multiprocessing import Process,Queue
from ikcp_gui2 import ikcp_gui

TxRx_interval = 0.1 # minimum period of Tx/Rx loop (sec)

def auto_detect_serial_port():
    ports = list(serial.tools.list_ports.comports())
    print("### Search results ###")
    for p in ports:
        print("   ",p)
        if 'USB Serial Port' in p.description:
            return p.device
    return None


def main():
    rx_list = []
    tx_list = []
    print()
    serial_port_name = None
    panel_file_list = []
    auto_prg_file_list = []
    com_print_flag = 0
    beep_flag = 0

    if len(sys.argv) >= 2:
        for file_name in sys.argv[1:]:
            if re.search(r'\.txt$',file_name):
                if os.path.exists(file_name):
                    panel_file_list.append(file_name)                    
                else:
                    print("### ERROR ### file",file_name,"not found")
                    print()
                    sys.exit()
            elif re.search(r'^(com)|(COM)[0-9]+$',file_name):
                serial_port_name = file_name
            elif re.search(r'\.py$',file_name):
                if os.path.exists(file_name):
                    auto_prg_file_list.append(file_name)
                else:
                    print("### ERROR ### file",file_name,"not found")
                    print()
                    sys.exit()
            elif re.search(r'^\-print$',file_name): # print auto_read/write to com window
                com_print_flag = 3
            elif re.search(r'^\-print_r',file_name): # print auto_read/write to com window
                com_print_flag = 1
            elif re.search(r'^\-print_w',file_name): # print auto_read/write to com window
                com_print_flag = 2
            elif re.search(r'^\-beep$',file_name): # beep on auto_read/write
                beep_flag = 3
            elif re.search(r'^\-beep_r',file_name): # beep on auto_read/write
                beep_flag = 1
            elif re.search(r'^\-beep_w',file_name): # beep on auto_read/write
                beep_flag = 2
            else:
                print("### ERROR ###",file_name,"? argument must be *.txt or *.py or com*")
                print()
                sys.exit()

    if not serial_port_name:
        print()
        serial_port_name = auto_detect_serial_port()

    if not serial_port_name:
        print()
        print("### ERROR ### USB-Serial port not recognized")
        print("    Please check the connection or specify the USB-Serial port name")
        print("    (ex. com3 or com4,...)")
        print()
        sys.exit()

    print()
    print("### Try to communicate",serial_port_name,"###")
    print()
    ser = serial.Serial(serial_port_name,57600,timeout=3)

    qtx = Queue()
    qrx = Queue()

    process = Process(target=ikcp_gui,args=(qtx,qrx,panel_file_list,auto_prg_file_list,com_print_flag,beep_flag))
    process.start()

    while True:
        if qtx.qsize() != 0:
            tx_list = qtx.get()
            if tx_list == [255,255]: # dicsonnect flag from GUI
                break
#            print("Tx:",tx_list)
            ser.write(tx_list)
            
        bytesWaiting = ser.inWaiting()
        if bytesWaiting != 0:
            onebyte = ser.read(1)
            oneint = int.from_bytes(onebyte,'big')
            rx_list.append(oneint)
            if oneint == 255: # find END code
#                print("  Rx:",rx_list)
                qrx.put(rx_list)
                rx_list = []


if __name__ == '__main__':
    main()
